@@ -1,13 +1,13 @@
-ArchiveGribGrab <- function(abbrev, model.date, model.run, pred, local.dir = ".", file.name = "fcst.grb",
+ArchiveGribGrab <- function(abbrev, model.date, model.run, preds, local.dir = ".", file.names = NULL,
     tidy = FALSE, verbose = TRUE, download.method = NULL, file.type = "grib2") {
     #Get archived grib file
     #INPUTS
     #    ABBREV - Model abbreviation returned by NOMADSArchiveList
     #    MODEL.DATE - The year. month, and day of the model run, in YYYYMMDD format
     #    MODEL.RUN - Which hour the model was run (i.e. 00, 06, 12, 18 for GFS)
-    #    PRED - Which prediction to get (analysis is 00)
+    #    PREDS - Which prediction to get (analysis is 00), a vector
     #    LOCAL.DIR is the directory to save the files in
-    #    FILE.NAME is the directory path and file name to save the grib file on disk, defaults to "fcst.grb" in current directory
+    #    FILE.NAMES is the directory path and file names to save the grib files on disk, defaults to "fcst.grb" in current directory
     #    TIDY asks whether to delete all grib files in the directory specified in FILE.NAME, default FALSE.
     #    This is useful to clear out previous model runs.
     #    It looks for all files named '.grb' and removes them.
@@ -21,6 +21,13 @@ ArchiveGribGrab <- function(abbrev, model.date, model.run, pred, local.dir = "."
     #        $FILE.NAME is the local file name where the data is stored
     #        $URL is the URL from which the file was downloaded
 
+   if(!is.null(file.names)) {
+      if(length(file.names) != length(preds)) {
+          stop("The length of the \"file.names\" argument is not the same as the length of the \"preds\" argument.  
+             Each file downloaded from NOMADS needs its own file name.")
+       }
+   }
+
    if(tidy) {
         unlink(list.files(local.dir, pattern = "*\\.grb$"))
    }
@@ -43,46 +50,56 @@ ArchiveGribGrab <- function(abbrev, model.date, model.run, pred, local.dir = "."
     #Get model info and set up URL to archive
     model.url <- NOMADSArchiveList("grib", abbrev=abbrev)$url
     download.url <- paste0(model.url, paste(model.date[1:6], collapse = ""), "/", paste(model.date, collapse = ""), "/") 
-    file.part <- paste0(paste(model.date, collapse = ""), "_", 
-    sprintf("%02.f", as.numeric(model.run)), 
-    "00_", sprintf("%03.f", as.numeric(pred)), suffix)
 
-    #Find out which grib files are in the archive
-    link.list <- unique(LinkExtractor(download.url))
-
-    #Check if the requested file is where we think it is
-    if(sum(grepl(paste0(".*", file.part, "$"), link.list)) < 1) {
-        stop(paste("The requested data file ending in", file.part, "does not appear to be in the archive. 
-            Try opening", download.url, "in a web browser to verify that it's missing."))
-    }
-
-    #Set up URL to file
-    grb.urls <- paste0(download.url, link.list[grepl(paste0(".*", file.part, "$"), link.list)])
-
-    #Download the file
-    if(length(grb.urls) > 1) {
-        warning("Two files were found:  ", paste(link.list[grepl(paste0(".*", file.part, "$"), link.list)], collapse = " "), ".  Both will be downloaded.")
-        c <- 1 
-    } else {
-        c <- "" 
-    }
-
-    use.curl <- FALSE #May use this as an option in the future
-    file.names <- NULL
-    for(grb.url in grb.urls) {
-        file.name.tmp <-  paste0(file.name, c)
-       if(is.null(download.method) & !use.curl) {#Let R decide how to download the file 
-           download.file(grb.url, paste(local.dir,file.name.tmp, sep = "/"), mode = "wb", quiet = !verbose)
+    grib.info <- NULL
+    for(k in 1:length(preds)) {
+        
+        if(is.null(file.names)) {
+            file.part <- paste0(paste(model.date, collapse = ""), "_", 
+                sprintf("%02.f", as.numeric(model.run)), 
+                "00_", sprintf("%03.f", as.numeric(preds[k])), suffix)
+            file.name <- file.part
+        } else {
+            file.name <- file.names[k]
+        } 
+        #Find out which grib files are in the archive
+        link.list <- unique(LinkExtractor(download.url))
+    
+        #Check if the requested file is where we think it is
+        if(sum(grepl(paste0(".*", file.part, "$"), link.list)) < 1) {
+            warning(paste("The requested data file ending in", file.part, "does not appear to be in the archive. 
+                Try opening", download.url, "in a web browser to verify that it's missing."))
+            next
         }
-        if(!is.null(download.method) & !use.curl) { #Download using specific method
-            download.file(grb.url, paste(local.dir,file.name.tmp, sep = "/"), download.method, mode = "wb", quiet = !verbose)
+    
+        #Set up URL to file
+        grb.urls <- paste0(download.url, link.list[grepl(paste0(".*", file.part, "$"), link.list)])
+    
+        #Download the file
+        if(length(grb.urls) > 1) {
+            warning("Two files were found:  ", paste(link.list[grepl(paste0(".*", file.part, "$"), link.list)], collapse = " "), ".  Both will be downloaded.")
+            c <- 1 
+        } else {
+            c <- "" 
         }
-        file.names <- append(file.names, file.name.tmp)
-        if(c != "") {
-            c <- c + 1
+    
+        use.curl <- FALSE #May use this as an option in the future
+        file.names <- NULL
+        for(grb.url in grb.urls) {
+            file.name.tmp <-  paste0(c, file.name)
+           if(is.null(download.method) & !use.curl) {#Let R decide how to download the file 
+               download.file(grb.url, paste(local.dir,file.name.tmp, sep = "/"), mode = "wb", quiet = !verbose)
+            }
+            if(!is.null(download.method) & !use.curl) { #Download using specific method
+                download.file(grb.url, paste(local.dir,file.name.tmp, sep = "/"), download.method, mode = "wb", quiet = !verbose)
+            }
+            file.names <- append(file.names, file.name.tmp)
+            if(c != "") {
+                c <- c + 1
+            }
         }
+        grib.info[[k]] <- list(local.dir = normalizePath(local.dir), file.name = file.names, url = grb.urls)
     }
-    grib.info <- list(local.dir = normalizePath(local.dir), file.name = file.names, url = grb.urls)
     return(grib.info)
 } 
 
@@ -106,18 +123,27 @@ CheckNOMADSArchive <- function(abbrev, model.date = NULL) {
         #Find out which months are available
         month.list <- grep("\\d{6}/", LinkExtractor(model.url), value = TRUE)
         for(month in month.list) {
-            date.list <- grep("\\d{8}/", LinkExtractor(paste0(model.url, month)), value = TRUE)
-            for(day in date.list) {
-                model.list <- append(model.list, grep("grb\\d?$", LinkExtractor(paste0(model.url, month, day)), value = TRUE))
-             }
+            month.url <- paste0(model.url, month)
+            if(RCurl::url.exists(month.url)) {
+                date.list <- grep("\\d{8}/", LinkExtractor(month.url), value = TRUE)
+                for(day in date.list) {
+                    day.url <- paste0(model.url, month, day)
+                    if(RCurl::url.exists(day.url)) {
+                        model.list <- append(model.list, grep("grb\\d?$", LinkExtractor(day.url), value = TRUE))
+                    }
+                }
+            }
          }
      } else {
             model.date <- as.numeric(strsplit(as.character(model.date), split = "")[[1]])
-            model.list <- append(
-                model.list, 
-                grep("grb\\d?$", 
-                LinkExtractor(paste0(model.url, paste(model.date[1:6], collapse = ""), "/", paste(model.date, collapse = ""), "/")),
-                value = TRUE))       
+            check.url <- paste0(model.url, paste(model.date[1:6], collapse = ""), "/", paste(model.date, collapse = ""), "/")
+            if(RCurl::url.exists(check.url)) {
+                model.list <- append(
+                    model.list, 
+                    grep("grb\\d?$", 
+                    LinkExtractor(check.url),
+                   value = TRUE))       
+            }
      }
 
      model.list <- as.vector(model.list)
@@ -127,6 +153,9 @@ CheckNOMADSArchive <- function(abbrev, model.date = NULL) {
          model.run = stringr::str_replace_all(stringr::str_extract(model.list, "_\\d{4}_"), "_", ""),
          pred = stringr::str_replace(stringr::str_replace(stringr::str_extract(model.list, "_\\d{3}\\."), "\\.", ""), "_", ""),
          file.name = model.list)
+     if(length(available.models$date) == 0) {
+        warning("No products are available for that model and date.  Sorry!")
+     }
      invisible(available.models)
         
 }
